searchData={"items":[{"type":"module","title":"ExCmd","doc":"ExCmd is an alternative for beam [ports](https://hexdocs.pm/elixir/Port.html)\nwith back-pressure and non-blocking IO.","ref":"ExCmd.html"},{"type":"module","title":"Quick Start - ExCmd","doc":"Run a command and read from stdout\n\n```\niex> ExCmd.stream!(~w(echo Hello))\n...> |> Enum.into(\"\") # collect as string\n\"Hello\\n\"\n```\n\nString input\n```\niex> ExCmd.stream!(~w(cat), input: \"Hello World\")\niex> |> Enum.into(\"\")\n\"Hello World\"\n```\n\nList of strings\n\n```\niex> ExCmd.stream!(~w(cat), input: [\"Hello\", \" \", \"World\"])\niex> |> Enum.into(\"\")\n\"Hello World\"\n```\n\nBinary data\n\n```\niex> ExCmd.stream!(~w(base64), input: <<1, 2, 3, 4, 5>>)\niex> |> Enum.into(\"\")\n\"AQIDBAU=\\n\"\n```\n\nIOData\n\n```\niex> ExCmd.stream!(~w(base64), input: [<<1, 2>>, [3], [<<4, 5>>]])\niex> |> Enum.into(\"\")\n\"AQIDBAU=\\n\"\n```\n\nRun a command with list of strings as input\n\n```\niex> ExCmd.stream!(~w(cat), input: [\"Hello\", \" \", \"World\"])\n...> |> Enum.into(\"\") # collect as string\n\"Hello World\"\n```\n\nRun a command with input as Stream\n\n```\niex> input_stream = Stream.map(1..10, fn num -> \"#{num} \" end)\niex> ExCmd.stream!(~w(cat), input: input_stream)\n...> |> Enum.into(\"\")\n\"1 2 3 4 5 6 7 8 9 10 \"\n```\n\nRun a command with input as infinite stream\n\n```\n# create infinite stream\niex> input_stream = Stream.repeatedly(fn -> \"A\" end)\niex> binary =\n...>   ExCmd.stream!(~w(cat), input: input_stream, ignore_epipe: true) # we need to ignore epipe since we are terminating the program before the input completes\n...>   |> Stream.take(2) # we must limit since the input stream is infinite\n...>   |> Enum.into(\"\")\niex> is_binary(binary)\ntrue\niex> \"AA\" <> _ = binary # we might get more than 2 \"A\" due to buffering\n```\n\nRun a command with input Collectable\n\n```\n# ExCmd calls the callback with a sink where the process can push the data\niex> ExCmd.stream!(~w(cat), input: fn sink ->\n...>   Stream.map(1..10, fn num -> \"#{num} \" end)\n...>   |> Stream.into(sink) # push to the external process\n...>   |> Stream.run()\n...> end)\n...> |> Stream.take(100) # we must limit since the input stream is infinite\n...> |> Enum.into(\"\")\n\"1 2 3 4 5 6 7 8 9 10 \"\n```\n\nWhen the command wait for the input stream to close\n\n```\n# base64 command wait for the input to close and writes data to stdout at once\niex> ExCmd.stream!(~w(base64), input: [\"abcdef\"])\n...> |> Enum.into(\"\")\n\"YWJjZGVm\\n\"\n```\n\n`stream!/2` raises non-zero exit as error\n\n```\niex> ExCmd.stream!([\"sh\", \"-c\", \"echo 'foo' && exit 10\"])\n...> |> Enum.to_list()\n** (ExCmd.Stream.AbnormalExit) program exited with exit status: 10\n```\n\n`stream/2` variant returns exit status as last element\n\n```\niex> ExCmd.stream([\"sh\", \"-c\", \"echo 'foo' && exit 10\"])\n...> |> Enum.to_list()\n[\n  \"foo\\n\",\n  {:exit, {:status, 10}} # returns exit status of the program as last element\n]\n```\n\nYou can fetch exit_status from the error for `stream!/2`\n\n```\niex> try do\n...>   ExCmd.stream!([\"sh\", \"-c\", \"exit 10\"])\n...>   |> Enum.to_list()\n...> rescue\n...>   e in ExCmd.Stream.AbnormalExit ->\n...>     e.exit_status\n...> end\n10\n```\n\nWith `max_chunk_size` set\n\n```\niex> data =\n...>   ExCmd.stream!(~w(cat /dev/urandom), max_chunk_size: 100, ignore_epipe: true)\n...>   |> Stream.take(5)\n...>   |> Enum.into(\"\")\niex> byte_size(data)\n500\n```\n\nWhen input and output run at different rate\n\n```\niex> input_stream = Stream.map(1..1000, fn num -> \"X #{num} X\\n\" end)\niex> ExCmd.stream!(~w(grep 250), input: input_stream)\n...> |> Enum.into(\"\")\n\"X 250 X\\n\"\n```\n\nWith stderr set to :disable (default)\n\n```\niex> ExCmd.stream!([\"sh\", \"-c\", \"echo foo; echo bar >&2\"])\n...> |> Enum.to_list()\n[\"foo\\n\"]\n```\n\nWith stderr set to :redirect_to_stdout\n\n```\niex> ExCmd.stream!([\"sh\", \"-c\", \"echo foo; echo bar >&2\"], stderr: :redirect_to_stdout)\n...> |> Enum.into(\"\")\n\"foo\\nbar\\n\"\n```\n\nWith stderr set to :console\n\n```\nExCmd.stream!([\"sh\", \"-c\", \"echo foo; echo bar >&2\"], stderr: :console)\n|> Enum.to_list()\n# STDERR output will be written to the console\n```\n\n\nFor more details about stream API, see `ExCmd.stream!/2` and `ExCmd.stream/2`.\n\nFor more details about inner working, please check `ExCmd.Process`\ndocumentation.","ref":"ExCmd.html#module-quick-start"},{"type":"type","title":"ExCmd.collectable_func/0","doc":"","ref":"ExCmd.html#t:collectable_func/0"},{"type":"function","title":"ExCmd.stream/2","doc":"Same as `ExCmd.stream!/2` but the program exit status is passed as last\nelement of the stream.\n\nThe last element will be of the form `{:exit, term()}`. `term` will be a\npositive integer in case of normal exit and `:epipe` in case of epipe error\n\nSee `ExCmd.stream!/2` documentation for details about the options and\nexamples.","ref":"ExCmd.html#stream/2"},{"type":"function","title":"ExCmd.stream!/2","doc":"Runs the command with arguments and return an the stdout as lazily\nEnumerable stream, similar to [`Stream`](https://hexdocs.pm/elixir/Stream.html).\n\nFirst parameter must be a list containing command with arguments.\nExample: `[\"cat\", \"file.txt\"]`.","ref":"ExCmd.html#stream!/2"},{"type":"function","title":"Options - ExCmd.stream!/2","doc":"* `input` - Input can be either an `Enumerable` or a function which accepts `Collectable`.\n    * String or Binary:\n      ```\n      # List\n      ExCmd.stream!(~w(cat), input: \"Hello World\") |> Enum.into(\"\")\n      # Stream\n      ExCmd.stream!(~w(cat), input: <<1, 2, 3, 4, 5>>) |> Enum.into(\"\")\n      ```\n\n    * Enumerable:\n\n      ```\n      # List\n      ExCmd.stream!(~w(base64), input: [\"hello\", \"world\"]) |> Enum.to_list()\n      # Stream\n      ExCmd.stream!(~w(cat), input: File.stream!(\"log.txt\", [], 65_531)) |> Enum.to_list()\n      ```\n\n    * Collectable:\n\n      If the input in a function with arity 1, ExCmd will call that function\n      with a `Collectable` as the argument. The function must *push* input to this\n      collectable. Return value of the function is ignored.\n\n      ```\n      ExCmd.stream!(~w(cat), input: fn sink -> Enum.into(1..100, sink, &to_string/1) end)\n      |> Enum.to_list()\n      ```\n\n      By defaults no input is sent to the command.\n\n  * `exit_timeout` - Duration to wait for external program to exit after completion\n(when stream ends). Defaults to `:infinity`\n\n  * `max_chunk_size` - Maximum size of iodata chunk emitted by the stream.\nChunk size can be less than the `max_chunk_size` depending on the amount of\ndata available to be read. Defaults to 65_531. Value must <= 65_531 and > 0.\n\n  * `stderr`  -  different ways to handle stderr stream.\n      1. `:console`  -  stderr output is redirected to console (Default)\n      2. `:redirect_to_stdout`  -  stderr output is redirected to stdout\n      3. `:disable`  -  stderr output is redirected `/dev/null` suppressing all output\n\n     See [`:stderr`](`m:ExCmd.Process#module-stderr`) for more details and issues associated with them.\n\n  * `ignore_epipe` - When set to true, reader can exit early without raising error.\nTypically writer gets `EPIPE` error on write when program terminate prematurely.\nWith `ignore_epipe` set to true this error will be ignored. This can be used to\nmatch UNIX shell default behaviour. EPIPE is the error raised when the reader finishes\nthe reading and close output pipe before command completes. Defaults to `false`.\n\nRemaining options are passed to `ExCmd.Process.start_link/2`\n\nIf program exits with non-zero exit status or :epipe then `ExCmd.Stream.AbnormalExit`\nerror will be raised with `exit_status` field set.","ref":"ExCmd.html#stream!/2-options"},{"type":"function","title":"Examples - ExCmd.stream!/2","doc":"```\nExCmd.stream!(~w(ffmpeg -i pipe:0 -f mp3 pipe:1), input: File.stream!(\"music_video.mkv\", [], 65_535))\n|> Stream.into(File.stream!(\"music.mp3\"))\n|> Stream.run()\n```\n\nStream with stderr redirected to stdout\n\n```\nExCmd.stream!([\"sh\", \"-c\", \"echo foo; echo bar >&2\"], stderr: :redirect_to_stdout)\n|> Stream.map(&IO.write/1)\n|> Stream.run()\n```","ref":"ExCmd.html#stream!/2-examples"},{"type":"module","title":"ExCmd.Process","doc":"GenServer which wraps spawned external command.\n\nUse `ExCmd.stream!/1` over using this. Use this only if you are\nfamiliar with life-cycle and need more control of the IO streams\nand OS process.","ref":"ExCmd.Process.html"},{"type":"module","title":"Comparison with Port - ExCmd.Process","doc":"* it is demand driven. User explicitly has to `read` the command\noutput, and the progress of the external command is controlled\nusing OS pipes. ExCmd never load more output than we can consume,\nso we should never experience memory issues\n\n  * it can close stdin while consuming output\n\n  * tries to handle zombie process by attempting to cleanup\nexternal process. Note that there is no middleware involved\nwith ex_cmd so it is still possible to endup with zombie process.\n\n  * selectively consume stdout and stderr\n\nInternally ExCmd uses non-blocking asynchronous system calls\nto interact with the external process. It does not use port's\nmessage based communication, instead uses raw stdio and NIF.\nUses asynchronous system calls for IO. Most of the system\ncalls are non-blocking, so it should not block the beam\nschedulers. Make use of dirty-schedulers for IO","ref":"ExCmd.Process.html#module-comparison-with-port"},{"type":"module","title":"Introduction - ExCmd.Process","doc":"`ExCmd.Process` is a process based wrapper around the external\nprocess. It is similar to `port` as an entity but the interface is\ndifferent. All communication with the external process must happen\nvia `ExCmd.Process` interface.\n\nExCmd process life-cycle tied to external process and owners. All\nsystem resources such are open file-descriptors, external process\nare cleaned up when the `ExCmd.Process` dies.","ref":"ExCmd.Process.html#module-introduction"},{"type":"module","title":"Owner - ExCmd.Process","doc":"Each `ExCmd.Process` has an owner. And it will be the process which\ncreated it (via `ExCmd.Process.start_link/2`). Process owner can not\nbe changed.\n\nOwner process will be linked to the `ExCmd.Process`. So when the\nex_cmd process is dies abnormally the owner will be killed too or\nvisa-versa. Owner process should avoid trapping the exit signal, if\nyou want avoid the caller getting killed, create a separate process\nas owner to run the command and monitor that process.\n\nOnly owner can get the exit status of the command, using\n`ExCmd.Process.await_exit/2`. All ex_cmd processes **MUST** be\nawaited.  Exit status or reason is **ALWAYS** sent to the owner. It\nis similar to [`Task`](https://hexdocs.pm/elixir/Task.html). If the\nowner exit without `await_exit`, the ex_cmd process will be killed,\nbut if the owner continue without `await_exit` then the ex_cmd\nprocess will linger around till the process exit.\n\n```\niex> alias ExCmd.Process\niex> {:ok, p} = Process.start_link(~w(echo hello))\niex> Process.read(p, 100)\n{:ok, \"hello\\n\"}\niex> Process.read(p, 100) # read till we get :eof\n:eof\niex> Process.await_exit(p)\n{:ok, 0}\n```","ref":"ExCmd.Process.html#module-owner"},{"type":"module","title":"Pipe & Pipe Owner - ExCmd.Process","doc":"Standard IO pipes/channels/streams of the external process such as\nSTDIN, STDOUT, STDERR are called as Pipes. User can either write or\nread data from pipes.\n\nEach pipe has an owner process and only that process can write or\nread from the ex_cmd process. By default the process who created the\nex_cmd process is the owner of all the pipes. Pipe owner can be\nchanged using `ExCmd.Process.change_pipe_owner/3`.\n\nPipe owner is monitored and the pipes are closed automatically when\nthe pipe owner exit. Pipe Owner can close the pipe early using\n`ExCmd.Process.close_stdin/1` etc.\n\n`ExCmd.Process.await_exit/2` closes all of the caller owned pipes by\ndefault.\n\n```\niex> {:ok, p} = Process.start_link(~w(cat))\niex> writer = Task.async(fn ->\n...>   :ok = Process.change_pipe_owner(p, :stdin, self())\n...>   Process.write(p, \"Hello World\")\n...> end)\niex> Task.await(writer)\n:ok\niex> Process.read(p, 100)\n{:ok, \"Hello World\"}\niex> Process.await_exit(p)\n{:ok, 0}\n```","ref":"ExCmd.Process.html#module-pipe-pipe-owner"},{"type":"module","title":"Pipe Operations - ExCmd.Process","doc":"Only Pipe owner can read or write date to the owned pipe.\nAll Pipe operations (read/write) blocks the caller as a mechanism\nto put back-pressure, and this also makes the API simpler.\nThis is same as how command-line programs works on the shell,\nalong with pipes in-between, Example: `cat larg-file | grep \"foo\"`.\nInternally ExCmd uses asynchronous IO APIs to avoid blocking VM\n(by default NIF calls blocks the VM scheduler),\nso you can open several pipes and do concurrent IO operations without\nblocking VM.\n\n\n### `stderr`\n\nby default is `:stderr` is connected to console, data written to\nstderr will appear on the console.\n\nYou can change the behavior by setting `:stderr`:\n\n  1. `:console`  -  stderr output is redirected to console\n  2. `:redirect_to_stdout`  -  stderr output is redirected to stdout\n  3. `:disable`  -  stderr output is redirected `/dev/null` suppressing all output. See below for more details. (Default)","ref":"ExCmd.Process.html#module-pipe-operations"},{"type":"module","title":"Using `redirect_to_stdout` - ExCmd.Process","doc":"stderr data will be redirected to stdout. When you read stdout\nyou will see both stdout & stderr combined and you won't be\nable differentiate stdout and stderr separately.\nThis is similar to `:stderr_to_stdout` option present in\n[Ports](https://www.erlang.org/doc/apps/erts/erlang.html#open_port/2).\n\n> #### Unexpected Behaviors {: .warning}\n>\n> On many systems, `stdout` and `stderr` are separated. And between\n> the source program to ExCmd, via the kernel, there are several places\n> that may buffer data, even temporarily, before ExCmd is ready\n> to read them. There is no enforced ordering of the readiness of\n> these independent buffers for ExCmd to make use of.\n>\n> This can result in unexpected behavior, including:\n>\n>  * mangled data, for example, UTF-8 characters may be incomplete\n> until an additional buffered segment is released on the same\n> source\n>  * raw data, where binary data sent on one source, is incompatible\n> with data sent on the other source.\n>  * interleaved data, where what appears to be synchronous, is not\n>\n> In short, the two streams might be combined at arbitrary byte position\n> leading to above mentioned issue.\n>\n> Most well-behaved command-line programs are unlikely to exhibit\n> this, but you need to be aware of the risk.\n>\n> A good example of this unexpected behavior is streaming JSON from\n> an external tool to ExCmd, where normal JSON output is expected on\n> stdout, and errors or warnings via stderr. In the case of an\n> unexpected error, the stdout stream could be incomplete, or the\n> stderr message might arrive before the closing data on the stdout\n> stream.","ref":"ExCmd.Process.html#module-using-redirect_to_stdout"},{"type":"module","title":"Process Termination - ExCmd.Process","doc":"When owner does (normally or abnormally) the ExCmd process always\nterminated irrespective of pipe status or process status. External\nprocess get a chance to terminate gracefully, if that fail it will\nbe killed.\n\nIf owner calls `await_exit` then the owner owned pipes are closed\nand we wait for external process to terminate, if the process\nalready terminated then call returns immediately with exit\nstatus. Else command will be attempted to stop gracefully following\nthe exit sequence based on the timeout value (5s by default).\n\nIf owner calls `await_exit` with `timeout` as `:infinity` then\nExCmd does not attempt to forcefully stop the external command and\nwait for command to exit on itself. The `await_exit` call can be blocked\nindefinitely waiting for external process to terminate.\n\nIf external process exit on its own, exit status is collected and\nExCmd process will wait for owner to close pipes. Most commands exit\nwith pipes are closed, so just ensuring to close pipes when works is\ndone should be enough.\n\nExample of process getting terminated by `SIGTERM` signal\n\n```\n# sleep command does not watch for stdin or stdout, so closing the\n# pipe does not terminate the sleep command.\niex> {:ok, p} = Process.start_link(~w(sleep 100000000)) # sleep indefinitely\niex> Process.await_exit(p, 2000) # ensure `await_exit` finish within `2000ms`. By default it waits for 5s\n{:error, :killed} # command exit due to SIGTERM\n```","ref":"ExCmd.Process.html#module-process-termination"},{"type":"module","title":"Examples - ExCmd.Process","doc":"Run a command without any input or output\n\n```\niex> {:ok, p} = Process.start_link([\"sh\", \"-c\", \"exit 2\"])\niex> Process.await_exit(p)\n{:ok, 2}\n```\n\nSingle process reading and writing to the command\n\n```\n# bc is a calculator, which reads from stdin and writes output to stdout\niex> {:ok, p} = Process.start_link(~w(cat))\niex> Process.write(p, \"hello\\n\") # there must be new-line to indicate the end of the input line\n:ok\niex> Process.read(p)\n{:ok, \"hello\\n\"}\niex> Process.write(p, \"world\\n\")\n:ok\niex> Process.read(p)\n{:ok, \"world\\n\"}\n# We must close stdin to signal the command that we are done.\n# since `await_exit` implicitly closes the pipes, in this case we don't have to\niex> Process.await_exit(p)\n{:ok, 0}\n```\n\nRunning a command which flush the output on stdin close. This is not\nsupported by Erlang/Elixir ports.\n\n```\n# `base64` command reads all input and writes encoded output when stdin is closed.\niex> {:ok, p} = Process.start_link(~w(base64))\niex> Process.write(p, \"abcdef\")\n:ok\niex> Process.close_stdin(p) # we can selectively close stdin and read all output\n:ok\niex> Process.read(p)\n{:ok, \"YWJjZGVm\\n\"}\niex> Process.read(p) # typically it is better to read till we receive :eof when we are not sure how big the output data size is\n:eof\niex> Process.await_exit(p)\n{:ok, 0}\n```\n\nRead and write to pipes in separate processes\n\n```\niex> {:ok, p} = Process.start_link(~w(cat))\niex> writer = Task.async(fn ->\n...>   :ok = Process.change_pipe_owner(p, :stdin, self())\n...>   Process.write(p, \"Hello World\")\n...>   # no need to close the pipe explicitly here. Pipe will be closed automatically when process exit\n...> end)\niex> reader = Task.async(fn ->\n...>   :ok = Process.change_pipe_owner(p, :stdout, self())\n...>   Process.read(p)\n...> end)\niex> :timer.sleep(500) # wait for the reader and writer to change pipe owner, otherwise `await_exit` will close the pipes before we change pipe owner\niex> Process.await_exit(p, :infinity) # let the reader and writer take indefinite time to finish\n{:ok, 0}\niex> Task.await(writer)\n:ok\niex> Task.await(reader)\n{:ok, \"Hello World\"}\n```","ref":"ExCmd.Process.html#module-examples"},{"type":"function","title":"ExCmd.Process.await_exit/2","doc":"Wait for the program to terminate and get exit status.\n\n**ONLY** the Process owner can call this function. And all ExCmd\n**process MUST** be awaited (Similar to Task).\n\nExCmd first politely asks the program to terminate by closing the\npipes owned by the process owner (by default process owner is the\npipes owner). Most programs terminates when standard pipes are\nclosed.\n\nIf you have changed the pipe owner to other process, you have to\nclose pipe yourself or wait for the program to exit.\n\nIf the program fails to terminate within the timeout (default 5s)\nthen the program will be killed using the exit sequence by sending\n`SIGTERM`, `SIGKILL` signals in sequence.\n\nWhen timeout is set to `:infinity` `await_exit` wait for the\nprograms to terminate indefinitely.\n\nFor more details check module documentation.","ref":"ExCmd.Process.html#await_exit/2"},{"type":"function","title":"ExCmd.Process.change_pipe_owner/3","doc":"Changes the Pipe owner of the pipe to specified pid.\n\nNote that currently any process can change the pipe owner.\n\nFor more details about Pipe Owner, please check module docs.","ref":"ExCmd.Process.html#change_pipe_owner/3"},{"type":"function","title":"ExCmd.Process.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExCmd.Process.html#child_spec/1"},{"type":"function","title":"ExCmd.Process.close_stderr/1","doc":"Closes external program's standard error pipe (stderr)\n\nOnly owner of the pipe can close the pipe. This call will return\nimmediately.","ref":"ExCmd.Process.html#close_stderr/1"},{"type":"function","title":"ExCmd.Process.close_stdin/1","doc":"Closes external program's standard input pipe (stdin).\n\nOnly owner of the pipe can close the pipe. This call will return\nimmediately.","ref":"ExCmd.Process.html#close_stdin/1"},{"type":"function","title":"ExCmd.Process.close_stdout/1","doc":"Closes external program's standard output pipe (stdout)\n\nOnly owner of the pipe can close the pipe. This call will return\nimmediately.","ref":"ExCmd.Process.html#close_stdout/1"},{"type":"type","title":"ExCmd.Process.exit_status/0","doc":"","ref":"ExCmd.Process.html#t:exit_status/0"},{"type":"function","title":"ExCmd.Process.os_pid/1","doc":"Returns OS pid of the command\n\nThis is meant only for debugging. Avoid interacting with the\nexternal process directly","ref":"ExCmd.Process.html#os_pid/1"},{"type":"type","title":"ExCmd.Process.pipe_name/0","doc":"","ref":"ExCmd.Process.html#t:pipe_name/0"},{"type":"function","title":"ExCmd.Process.read/2","doc":"Returns bytes from executed command's stdout with maximum size `max_size`.\n\nBlocks if no data present in stdout pipe yet. And returns as soon as\ndata of any size is available.\n\nNote that `max_size` is the maximum size of the returned data. But\nthe returned data can be less than that depending on how the program\nflush the data etc.","ref":"ExCmd.Process.html#read/2"},{"type":"type","title":"ExCmd.Process.recv_commands/0","doc":"","ref":"ExCmd.Process.html#t:recv_commands/0"},{"type":"function","title":"ExCmd.Process.start_link/2","doc":"Starts `ExCmd.Process` server.\n\nStarts external program using `cmd_with_args` with options `opts`\n\n`cmd_with_args` must be a list containing command with arguments.\nexample: `[\"cat\", \"file.txt\"]`.","ref":"ExCmd.Process.html#start_link/2"},{"type":"function","title":"Options - ExCmd.Process.start_link/2","doc":"* `cd`   -  the directory to run the command in\n\n  * `env`  -  a list of tuples containing environment key-value.\nThese can be accessed in the external program\n\n  * `stderr`  -  different ways to handle stderr stream.\n      1. `:console`  -  stderr output is redirected to console\n      2. `:redirect_to_stdout`  -  stderr output is redirected to stdout\n      3. `:disable`  -  stderr output is redirected `/dev/null` suppressing all output (Default)\n\n    See [`:stderr`](#module-stderr) for more details and issues associated with them\n\nCaller of the process will be the owner owner of the ExCmd Process.\nAnd default owner of all opened pipes.\n\nPlease check module documentation for more details","ref":"ExCmd.Process.html#start_link/2-options"},{"type":"type","title":"ExCmd.Process.t/0","doc":"","ref":"ExCmd.Process.html#t:t/0"},{"type":"function","title":"ExCmd.Process.write/2","doc":"Writes iodata `data` to external program's standard input pipe.\n\nThis call blocks when the pipe is full. Returns `:ok` when\nthe complete data is written.","ref":"ExCmd.Process.html#write/2"},{"type":"module","title":"ExCmd.Stream","doc":"Defines a `ExCmd.Stream` struct returned by `ExCmd.stream!/2`.","ref":"ExCmd.Stream.html"},{"type":"type","title":"ExCmd.Stream.t/0","doc":"Struct members are private, do not depend on them","ref":"ExCmd.Stream.html#t:t/0"},{"type":"exception","title":"ExCmd.Process.Error","doc":"","ref":"ExCmd.Process.Error.html"},{"type":"exception","title":"ExCmd.Stream.AbnormalExit","doc":"","ref":"ExCmd.Stream.AbnormalExit.html"},{"type":"extras","title":"ExCmd","doc":"# ExCmd\n\n[![CI](https://github.com/akash-akya/ex_cmd/actions/workflows/elixir.yml/badge.svg)](https://github.com/akash-akya/ex_cmd/actions/workflows/elixir.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_cmd.svg)](https://hex.pm/packages/ex_cmd)\n[![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/ex_cmd/)\n\nExCmd is an Elixir library for running and communicating with external programs using a back-pressure mechanism. It provides a robust alternative to Elixir's built-in [Port](https://hexdocs.pm/elixir/Port.html) with improved memory management through demand-driven I/O.","ref":"readme.html"},{"type":"extras","title":"The Port I/O Problem - ExCmd","doc":"When using Elixir's built-in [Port](https://hexdocs.pm/elixir/Port.html), running external programs that generate large amounts of output (like streaming video using `ffmpeg`) can quickly lead to memory issues. This happens because Port I/O is not demand-driven - it consumes output from stdout as soon as it's available and sends it to the process mailbox. Since BEAM process mailboxes are unbounded, the output accumulates there waiting to be received.","ref":"readme.html#the-port-i-o-problem"},{"type":"extras","title":"Memory Usage Comparison - ExCmd","doc":"Let's look at how ExCmd handles memory compared to Port when processing large streams:\n\nUsing Port (memory grows unbounded):\n```elixir\nPort.open({:spawn_executable, \"/bin/cat\"}, [{:args, [\"/dev/random\"]}, {:line, 10}, :binary, :use_stdio])\n```\n\n![Port memory consumption](./images/port.png)\n\nUsing ExCmd (memory remains stable):\n```elixir\nExCmd.stream!(~w(cat /dev/random))\n|> Enum.each(fn data ->\n  IO.puts(IO.iodata_length(data))\nend)\n```\n\n![ExCmd memory consumption](./images/ex_cmd.png)\n\nExCmd solves this by implementing:\n- Demand-driven I/O with proper back-pressure\n- Efficient use of OS-backed stdio buffers\n- Stream-based API that integrates with Elixir's ecosystem","ref":"readme.html#memory-usage-comparison"},{"type":"extras","title":"Key Features - ExCmd","doc":"- **Back-pressure Support**: Controls data flow between your application and external programs\n- **Stream Abstraction**: Seamless integration with Elixir's Stream API\n- **Memory Efficient**: Demand-driven I/O prevents memory issues with large data transfers\n- **Cross-platform**: Pre-built binaries for MacOS, Windows, and Linux\n- **Process Management**: Proper program termination with no zombie processes\n- **Selective I/O Control**: Ability to close stdin while keeping stdout open\n- **No Dependencies**: No separate middleware or shim installation required","ref":"readme.html#key-features"},{"type":"extras","title":"Installation - ExCmd","doc":"Add `ex_cmd` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_cmd, \"~> x.x.x\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start Examples - ExCmd","doc":"","ref":"readme.html#quick-start-examples"},{"type":"extras","title":"Basic Command Execution - ExCmd","doc":"```elixir\n# Simple command execution\nExCmd.stream!(~w(echo Hello))\n|> Enum.into(\"\")\n# => \"Hello\\n\"\n\n# Get your IP address\nExCmd.stream!(~w(curl ifconfig.co))\n|> Enum.into(\"\")\n```","ref":"readme.html#basic-command-execution"},{"type":"extras","title":"Working with Input Streams - ExCmd","doc":"```elixir\n# String input\nExCmd.stream!(~w(cat), input: \"Hello World\")\n|> Enum.into(\"\")\n# => \"Hello World\"\n\n# List of strings\nExCmd.stream!(~w(cat), input: [\"Hello\", \" \", \"World\"])\n|> Enum.into(\"\")\n# => \"Hello World\"\n\n# Binary data\nExCmd.stream!(~w(base64), input: <<1, 2, 3, 4, 5>>)\n|> Enum.into(\"\")\n# => \"AQIDBAU=\\n\"\n\n# IOData\nExCmd.stream!(~w(base64), input: [<<1, 2>>, [3], [<<4, 5>>]])\n|> Enum.into(\"\")\n# => \"AQIDBAU=\\n\"\n```","ref":"readme.html#working-with-input-streams"},{"type":"extras","title":"Media Processing Examples - ExCmd","doc":"```elixir\n# Extract audio from video with controlled memory usage\nExCmd.stream!(~w(ffmpeg -i pipe:0 -f mp3 pipe:1),\n  input: File.stream!(\"music_video.mkv\", [], 65536))\n|> Stream.into(File.stream!(\"music.mp3\"))\n|> Stream.run()\n\n# Process video streams efficiently\nExCmd.stream!(~w(ffmpeg -i pipe:0 -c:v libx264 -f mp4 pipe:1),\n  input: File.stream!(\"input.mp4\", [], 65536),\n  max_chunk_size: 65536)\n|> Stream.into(File.stream!(\"output.mp4\"))\n|> Stream.run()\n```","ref":"readme.html#media-processing-examples"},{"type":"extras","title":"Error Handling - ExCmd","doc":"```elixir\n# stream!/2 raises on non-zero exit status\nExCmd.stream!([\"sh\", \"-c\", \"exit 10\"])\n|> Enum.to_list()\n# => ** (ExCmd.Stream.AbnormalExit) program exited with exit status: 10\n\n# stream/2 returns exit status as last element\nExCmd.stream([\"sh\", \"-c\", \"echo 'foo' && exit 10\"])\n|> Enum.to_list()\n# => [\"foo\\n\", {:exit, {:status, 10}}]\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Advanced Features - ExCmd","doc":"```elixir\n# Redirect stderr to stdout\nExCmd.stream!([\"sh\", \"-c\", \"echo foo; echo bar >&2\"],\n  stderr: :redirect_to_stdout)\n|> Enum.into(\"\")\n# => \"foo\\nbar\\n\"\n```","ref":"readme.html#advanced-features"},{"type":"extras","title":"Alternatives - ExCmd","doc":"- For NIF-based solutions without middleware overhead, consider [Exile](https://github.com/akash-akya/exile)\n- For simple command execution without streaming, Elixir's built-in Port might be sufficient","ref":"readme.html#alternatives"},{"type":"extras","title":"Documentation - ExCmd","doc":"Detailed documentation is available at [HexDocs](https://hexdocs.pm/ex_cmd/).","ref":"readme.html#documentation"},{"type":"extras","title":"License - ExCmd","doc":"See [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2020 Akash Hiremath\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}